# SPDX-FileCopyrightText: WTF Kooperative eG <https://wtf-eg.de/>
#
# SPDX-License-Identifier: AGPL-3.0-or-later

from flask import make_response, request

from ki.models import Profile, ProfileSkill, Skill, ProfileLanguage, Language


def find_profiles():
    page = int(request.args.get("page", 1))

    if page < 1:
        return make_response({"messages": {"page": "Die angefragte Seite muss mindestens 1 sein"}}, 400)

    page_size = int(request.args.get("page_size", 20))

    if page_size > 100:
        return make_response({"messages": {"page_size": "Die maximale Anzahl Einträge pro Seite beträgt 100"}}, 400)

    query = Profile.query.filter(Profile.visible.is_(True)) \
        .join(Profile.skills, isouter=True).join(ProfileSkill.skill, isouter=True) \
        .join(Profile.languages, isouter=True).join(ProfileLanguage.language, isouter=True)

    if "search" in request.args:
        terms = request.args["search"].split(" ")
        for term in terms:
            query = query.filter(
                Profile.nickname.like(f"%{term}%") | Skill.name.like(f"%{term}%") | Language.name.like(f"%{term}%"))

    if "nickname" in request.args:
        nickname = request.args.get("nickname")
        query = query.filter(Profile.nickname.like(f"%{nickname}%"))

    count = query.distinct(Profile.id).count()

    offset = (page - 1) * page_size
    db_profiles = query.limit(page_size).offset(offset).all()
    api_profiles = []

    for db_profile in db_profiles:
        api_profiles.append(db_profile.to_dict())

    return make_response({"total": count, "profiles": api_profiles})
